/*
 *      main.c
 *      
 *      Copyright 2008 Tuxgasy <tuxgasy@tuxfamily.org>
 *      
 *      This program is free software; you can redistribute it and/or modify
 *      it under the terms of the GNU General Public License as published by
 *      the Free Software Foundation; either version 2 of the License, or
 *      (at your option) any later version.
 *      
 *      This program is distributed in the hope that it will be useful,
 *      but WITHOUT ANY WARRANTY; without even the implied warranty of
 *      MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *      GNU General Public License for more details.
 *      
 *      You should have received a copy of the GNU General Public License
 *      along with this program; if not, write to the Free Software
 *      Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston,
 *      MA 02110-1301, USA.
 */

#include "main.h"
#include "listProducts.h"
#include "interface.h"
#include "callbacks.h"
#include "config.h"
#include "preferences.h"
#include "print.h"

int main(int argc, char **argv){
	glda *pGlda;
	printf("=== glda %s ===\n", VERSION_GLDA);
	
	/* init libxml */
	printf("init libxml...");
	xmlXPathInit();
	printf("ok\n");
	
	/* init Gtk+ */
	printf("init gtk+...");
	gtk_init(&argc, &argv);
	printf("ok\n");
	
	/* init glda */
	printf("init Glda...");
	pGlda = initGlda(argv[1]);
	printf("ok\n");
	
	/* init Window */
	printf("init Window...\n");
	initWindow(pGlda);
	
	if(pGlda->filesave){
		if(!addListProductFromXml(pGlda)){
			pGlda->filesave = NULL;
			pGlda->isSave = FALSE;
		}
	}
	
	/* Show window */
	printf("Show the window...");
	gtk_widget_show_all(pGlda->pWindow);
	printf("ok\n");
	
	gtk_main();
	
	/* Free glda */
	printf("Free Glda...");
	freeGlda(pGlda);
	printf("ok\n");
	
	/* bye */
	printf("Bye ;)\n\n");
	
	return EXIT_SUCCESS;
}

glda* initGlda(gchar *filename){
	glda *pGlda;
	gchar *pText;
	gchar *content;
	gchar *fileToCopy;
	
	pGlda = (glda *)malloc(sizeof(glda));
	
	/* init user's config */
	pGlda->pConfig = initConfig();
	
	/* init the list of products */
	pText = g_key_file_get_string(pGlda->pConfig, "file", "file_list_product", NULL);
	pGlda->xmlDocListProducts = initListProducts(pText);
	
	if(!checkCompatibilityListProducts(pGlda->xmlDocListProducts, pText)){
		
		/* copy the file before modify */
		fileToCopy = g_strdup_printf("%s.bak", pText);
		g_file_get_contents(pText, &content, NULL, NULL);
		g_file_set_contents(fileToCopy, content, -1, NULL);
		g_free(fileToCopy);
		g_free(content);
		
		makeListProductsCompatible(pGlda->xmlDocListProducts, pText);
		
		/* reload the file */
		freeListProducts(pGlda->xmlDocListProducts);
		pGlda->xmlDocListProducts = initListProducts(pText);
	}
	
	/* Recent open file */
	pGlda->pManager = gtk_recent_manager_get_default();
	
	pGlda->pTextStatusbar = g_strdup_printf("Glda (Gtk Liste D'Achat) version %s", VERSION_GLDA);
	
	pGlda->nbTotalProducts = 0;
	pGlda->priceTotalProducts = 0;
	
	if(filename){
		pGlda->filesave = g_strdup_printf("%s", filename);
		gtk_recent_manager_add_item(pGlda->pManager, pGlda->filesave);
		pGlda->isSave = TRUE;
	}else{
		pGlda->filesave = g_key_file_get_string(pGlda->pConfig, "file", "last_list_opened", NULL);
		pGlda->isSave = TRUE;
		if(pGlda->filesave && strcmp(pGlda->filesave, "") == 0){
			g_free(pGlda->filesave);
			pGlda->filesave = NULL;
			pGlda->isSave = FALSE;
		}
	}
	
	return pGlda;
}

void freeGlda(glda *pGlda){
	gchar *fileUserConfig;
	
	/* save last list opened */
	fileUserConfig = g_strdup_printf("%s%s%s", getenv("HOME"), DIR_USER_CONFIG, FILE_USER_CONFIG);
	if(pGlda->filesave){
		g_key_file_set_string(pGlda->pConfig, "file", "last_list_opened", pGlda->filesave);
	}else{
		g_key_file_set_string(pGlda->pConfig, "file", "last_list_opened", "");
	}
	saveConfig(pGlda->pConfig, fileUserConfig);
	g_free(fileUserConfig);
	
	if(pGlda == NULL){
		return;
	}
	
	if(pGlda->xmlDocListProducts != NULL){
		freeListProducts(pGlda->xmlDocListProducts);
	}
	
	if(pGlda->pTextStatusbar != NULL){
		g_free(pGlda->pTextStatusbar);
	}
	
	free(pGlda);
}
